# pluralizers that are not so complex, they have an ending and no exceptions

def pluralize_oren(singular: str)-> str:
    if not singular.endswith("or"):
        return None
    
    return singular + "en"


def singularize_oren(plural:str)->str:

    # make sure we don't get too many false positive
    if not plural.endswith("oren") or len(plural) < 8:
        return None

    return plural[0:-2]


def pluralize_heid(singular: str) -> str:
    if not singular.endswith("heid"):
        return None
    
    return singular[0:-2] + "den"


def singularize_heid(plural: str) -> str:
    if not plural.endswith("heden"):
        return None
    
    return plural[0:-5] + "heid"


def pluralize_lui(singular: str) -> str:
    if not singular.endswith("lui"):
        return None
    
    return singular[0:-2] + "ieden"


def pluralize_lui(singular: str) -> str:
    if not singular.endswith("lui"):
        return None

    return singular[0:-2] + "ieden"


def singularize_jes(plural: str) -> str:

    if not plural.endswith("jes"):
        return None

    return plural[0:-1]


from .helpers import change_last_character, create_ends_with_regex, VOWELS
import re

def pluralize_with_en(singular: str) -> str:
    singular = change_last_character(singular)

    if singular.endswith("ie"):
        return singular[0:-1] + "ën"
    
    if singular.endswith("e"):
        return singular + "ën"

    return singular + "en"


__xxx = re.compile("(^|[^aeoiu])[aeoiu][^aeoiu]$")


def singularize_with_trema_en(plural: str)->str:

    if plural.endswith("iën"):
        return plural[0:-2] + "e"

    if plural.endswith("ën"):
        return plural[0:-2]

    return None


__aaa = create_ends_with_regex("weg", "vat", "schot", "gat", "drag", "bad", "dag", "bevel")

def singularize_with_en_single_vowel(plural: str) -> str:

    if not plural.endswith("en"):
        return None

    singular = plural[0:-2]
    if not __aaa.search(singular):
        return None

    if singular.endswith("v"):
        return singular[0:-1] + "f"

    if singular.endswith("z"):
        return singular[0:-1] + "s"

    return singular


def singularize_with_en_double_vowel(plural: str) -> str:

    if not plural.endswith("en"):
        return None

    # ogen => oog
    singular = plural[0:-2]
    if __xxx.search(singular):
        singular = singular[0:-1] + singular[-2] + singular[-1]

        if singular.endswith("v"):
            return singular[0:-1] + "f"
        if singular.endswith("z"):
            return singular[0:-1] + "s"

        return singular


    return None

def singularize_with_en_double_consonant(plural: str) -> str:

    if not plural.endswith("en"):
        return None

    singular = plural[0:-2]
    if singular[-1] not in VOWELS and \
        singular[-1] == singular[-2] and \
        singular[-3] in VOWELS:
        return singular[0:-1]

    return None


import argparse
from pprint import pprint
from . import pluralize, pluralize_advanced, singularize, could_be_plural, singularize_advanced

def print_result(answer: str, verbose: bool, action: str, word: str):
    if answer:
        print(answer)
    elif verbose:
        print(f"Could not {action} '{word}'.")


def main():

    parser = argparse.ArgumentParser(description='Generates Dutch plural and singular nouns in a very imperfect way using Hunspell dictionaries. Why imperfect? Because the Dutch language is full of exceptions.')
    parser.add_argument('word', help='The word.')
    parser.add_argument("-p",  "--pluralize", help="pluralizes the word.", action="store_true")
    parser.add_argument("-pa", "--pluralize_advanced", help="shows advanced pluralization output.", action="store_true")
    parser.add_argument("-s",  "--singularize", help="singularizes the word.", action="store_true")
    parser.add_argument("-sa", "--singularize_advanced", help="shows advanced singularization output.", action="store_true")
    parser.add_argument("-v",  "--verbose", help="Shows an error message when a word could not be processed.", action="store_true")

    args = parser.parse_args()

    word = args.word

    do_pluralize = args.pluralize or (not args.singularize and not args.pluralize_advanced)

    if do_pluralize:
        answer = pluralize(word)
        print_result(answer, args.verbose, "pluralize", word)
    
    if args.singularize:
        answer = singularize(word)
        print_result(answer, args.verbose, "singularize", word)
    
    if args.pluralize_advanced:
        answer = pluralize_advanced(word)
        pprint(vars(answer))

    if args.singularize_advanced:
        answer = singularize_advanced(word)
        pprint(vars(answer))


if __name__ == "__main__":
    main()


from typing import Tuple

from hunspell import Hunspell

from .pluralizer import pluralize
from .speller import ensure_hunspell_nl
from .strategies.helpers import VOWELS, create_ends_with_regex
from .strategies.pluralize_by_hard_map import singularize_by_hard_map
from .strategies.pluralize_by_latin import singularize_by_latin
from .strategies.pluralize_eren import singularize_eren
from .strategies.pluralize_with_en import (
    singularize_with_en_double_consonant, singularize_with_en_double_vowel,
    singularize_with_en_single_vowel, singularize_with_trema_en)
from .strategies.pluralize_with_s import singularize_with_s
from .strategies.simple import (singularize_heid, singularize_jes,
                                singularize_oren)
                            
from .mapping import NounEndingMap

__possible_plural_endings = create_ends_with_regex(
    "en", 
    # heden
    "'s",
    "s",
    "ën",
    "a",
    "i"
)


def could_be_plural(plural: str, ending_overrides: NounEndingMap = None) -> bool:
    if __possible_plural_endings.search(plural):
        return True
    
    if ending_overrides:
        for key in ending_overrides.get_singular_map():
            if plural.endswith(key):
                return True

    return False


def __process_methods(plural: str, *args) -> [str]:

    options = list()
    for method in args:
        x = method(plural)
        if x:
            options.append(x)

    return options


def __stem(speller: Hunspell, plural:str) -> [str]:

    stems = list()
    for stem in speller.stem(plural):
        stem = stem.replace("ĳ", "ij")
        if len(plural) - len(stem) <= 3:
            ps = pluralize(stem)
            if ps == plural:
                stems.append(stem)

    return stems


class AdvancedSingularizationResult:

    def __init__(
        self,
        algorithic_singular: [str],
        singular: str,
        suggestions: Tuple[str],
        hunspell_spelled: bool,
        could_be_plural: bool,
    ):
        self.algorithic_singular = algorithic_singular
        self.singular = singular
        self.suggestions = suggestions
        self.hunspell_spelled = hunspell_spelled
        self.could_be_plural = could_be_plural


def singularize_advanced(plural: str, speller: Hunspell = None, ending_overrides: NounEndingMap = None) -> AdvancedSingularizationResult:

    if not could_be_plural(plural, ending_overrides):
        return AdvancedSingularizationResult(None, None, (), False, False)

    options = __process_methods(
        plural,
        lambda l: singularize_by_hard_map(l, ending_overrides), # should always be first!
        singularize_oren,
        singularize_heid,
        singularize_eren,
        singularize_jes,
        singularize_by_latin,
        singularize_with_s,
        singularize_with_trema_en,
        singularize_with_en_single_vowel,
        singularize_with_en_double_vowel,
        singularize_with_en_double_consonant
    )

    # debug
    # print("options", options)
    
    if not speller:
        speller = ensure_hunspell_nl()

    stems = __stem(speller, plural)

    # return option that is spelled correct
    for option in options:
        if speller.spell(option):
            return AdvancedSingularizationResult(options, option, (), True, True)

    stems = __stem(speller, plural)

    # debug
    # print("options", options, "stems", stems)

    if stems:
        return AdvancedSingularizationResult(options, stems[0], stems, True, True)

    if plural.endswith("'s"):
        singular = plural[0:-2]
        return create_final_options(singular, speller, options, stems)

    if plural.endswith("s"):
        singular = plural[0:-1]
        return create_final_options(singular, speller, options, stems)

    if plural.endswith("en"):
        singular = plural[0:-2]

        if singular.endswith("v"):
            singular = singular[0:-1] + "f"
        elif singular.endswith("z"):
            singular = singular[0:-1] + "s"

        return create_final_options(singular, speller, options, stems)

    return AdvancedSingularizationResult(options, None, stems, False, True)


def singularize(plural: str, speller: Hunspell = None, ending_overrides: NounEndingMap = None) -> str:
    return singularize_advanced(plural, speller, ending_overrides).singular


def create_final_options(singular: str, speller: Hunspell, options:[str], stems:[str]) -> AdvancedSingularizationResult:

    options.append(singular)

    if speller.spell(singular):
        return AdvancedSingularizationResult(options, singular, stems, True, True)
    else:
        return AdvancedSingularizationResult(options, None, stems, False, True)


import re

from .helpers import VOWELS, change_last_character, create_ends_with_regex

__eren_endings = create_ends_with_regex("kind", "ei", "been", "blad", "kalf", "lam", "lied", "rund", "goed", "gemoed", "hoen", "rad", "gelid")
__deren_endings = create_ends_with_regex("been", "hoen")
__excetions_last_character_changed = create_ends_with_regex('blad', 'lied', 'rad', 'goed', 'gemoed')


def pluralize_eren(singular: str) -> str:

    if not __eren_endings.search(singular):
        return None

    if __deren_endings.search(singular):
        singular += 'd'

    if not __excetions_last_character_changed.search(singular):
        singular = change_last_character(singular)

    return singular + 'eren'


def singularize_eren(plural: str) -> str:

    if len(plural) < 6:
        return None

    if plural.endswith("eren"):
        singular = plural[0:-4]

        if singular.endswith("d"):
            minus_d = singular[0:-1]
            if __deren_endings.search(minus_d):
                return minus_d

        if singular.endswith("v"):
            return singular[0:-1] + "f"

        if singular[-1] not in VOWELS and \
            singular[-1] == singular[-2]:
            return singular[0:-1]

        return singular


    return None



__lui_endings = []
__lieden_endings = ['veer', 'handwerks', 'boots', 'boots', 'speel']
__en_endings = ["Frans", "Engels", "frans", "engels", "mens"]

def pluralize_man(singlar: str) -> str:
    if not singlar.endswith("man"):
        return None

    root = singlar[0:-3]

    if root in __lui_endings:
        return root + "lui"

    if root in __lieden_endings:
        return root + "lieden"

    if root in __en_endings:
        return root + "en"

    return None


import re

__ignored_ending = re.compile("(bevel|spel|ambtenaar)$")
__endings_with_no_preceeding_vowel = re.compile("(?<![aeiou])(el|em|en|er|aar|aard|erd)$")
__endings = re.compile("(ster|je)$")

def pluralize_with_s(singular: str) -> str:

    if __ignored_ending.search(singular):
        return None
    
    if __endings_with_no_preceeding_vowel.search(singular) or __endings.search(singular):
        return singular + "s"

    return None


def singularize_with_s(plural: str) -> str:

    if plural.endswith("s"):
        singular = plural[0:-1]
        if plural == pluralize_with_s(singular):
            return singular

    if plural.endswith("'s"):
        singular = plural[0:-2]
        if plural == pluralize_with_s(singular):
            return singular
    
    return None

    



from hunspell import Hunspell

from .mapping import NounEndingMap
from .pluralizer import (AdvancedPluralizationResult, pluralize,
                         pluralize_advanced)
from .singularizer import (AdvancedSingularizationResult, could_be_plural,
                           singularize, singularize_advanced)
from .speller import ensure_hunspell_nl


class Pluralizer:

    def __init__(self, speller: Hunspell = None, ending_overrides: NounEndingMap = None):
        super().__init__()

        self.__speller = speller or ensure_hunspell_nl()
        self.__ending_overrides = ending_overrides


    def add_ending_overrides(self, ending_overrides: NounEndingMap):

        if ending_overrides:
            if(self.__ending_overrides == None):
                self.__ending_overrides = NounEndingMap()

            self.__ending_overrides.add_range(
                ending_overrides.get_plural_map())


    def singularize(self, plural: str) -> str:
        return singularize(plural, self.__speller, self.__ending_overrides)


    def singularize_advanced(self, plural: str) -> AdvancedSingularizationResult:
        return singularize_advanced(plural, self.__speller, self.__ending_overrides)


    def could_be_plural(self, plural: str) -> bool:
        return could_be_plural(plural, self.__ending_overrides)


    def pluralize(self, singular: str) -> str:
        return pluralize(singular, self.__speller, self.__ending_overrides)


    def pluralize_advanced(self, singular: str) -> AdvancedPluralizationResult:
        return pluralize_advanced(singular, self.__speller, self.__ending_overrides)



__version__ = "0.0.28"

__all__ = ['pluralize', 'pluralize_advanced', 'singularize',
           'could_be_plural', 'singularize_advanced', 'NounEndingMap', 'Pluralizer']

from hunspell import Hunspell

from .mapping import NounEndingMap
from .pluralizer import pluralize, pluralize_advanced
from .pluralizer_cls import Pluralizer
from .singularizer import could_be_plural, singularize, singularize_advanced



import re

from .helpers import create_ends_with_regex

__sen_endings = create_ends_with_regex('catalogus')

def pluralize_by_latin(singular: str) -> str:
    if singular.endswith("aus"):
        return None
    if singular.endswith("um"):
        return singular[0:-2] + "a"
    if __sen_endings.search(singular):
        return singular + "sen"
    if singular.endswith("cus"):
        return singular[0:-3] + "ci"
    if singular.endswith("us"):
        return singular[0:-2] + "i"
    return None


def singularize_by_latin(plural: str) -> str:
    
    if plural.endswith("a"):
        return plural[0:-1] + "um"

    if plural.endswith("ci"):
        return plural[0:-2] + "cus"

    if plural.endswith("i"):
        return plural[0:-1] + "us"

    return None



import re
from typing import Tuple

from hunspell import Hunspell

from .mapping import NounEndingMap
from .replacements import (SearchResult, search_by_dictionary,
                           search_by_dictionary_plus_s, search_by_suggestions)
from .speller import ensure_hunspell_nl, get_plural_nouns
from .strategies.pluralize_bastard import pluralize_bastard
from .strategies.pluralize_by_hard_map import pluralize_by_hard_map
from .strategies.pluralize_by_latin import pluralize_by_latin
from .strategies.pluralize_eren import pluralize_eren
from .strategies.pluralize_long_vowel import pluralize_long_vowel
from .strategies.pluralize_man import pluralize_man
from .strategies.pluralize_with_en import pluralize_with_en
from .strategies.pluralize_with_s import pluralize_with_s
from .strategies.simple import pluralize_heid, pluralize_lui, pluralize_oren


def __pluralize(singular: str, ending_overrides: NounEndingMap = None) -> str:
    return \
        pluralize_by_hard_map(singular, ending_overrides) or \
        pluralize_eren(singular) or \
        pluralize_oren(singular) or \
        pluralize_heid(singular) or \
        pluralize_lui(singular) or \
        pluralize_man(singular) or \
        pluralize_by_latin(singular) or \
        pluralize_long_vowel(singular) or \
        pluralize_bastard(singular) or \
        pluralize_with_s(singular) or \
        pluralize_with_en(singular) or \
        None


class AdvancedPluralizationResult:

    def __init__(
        self,
        algorithmic_plural: str,
        plural: str,
        suggestions: Tuple[str],
        switched_ending_from: str,
        switched_ending_to: str,
        hunspell_spelled: bool
    ):
        self.algorithmic_plural = algorithmic_plural
        self.plural = plural
        self.suggestions = suggestions
        self.switched_ending_from = switched_ending_from
        self.switched_ending_to = switched_ending_to
        self.hunspell_spelled = hunspell_spelled


def pluralize_advanced(singular: str, speller: Hunspell = None, ending_overrides: NounEndingMap = None) -> AdvancedPluralizationResult:

    if not speller:
        speller = ensure_hunspell_nl()

    plural = __pluralize(singular, ending_overrides)

    # empty plural - just stop
    if not plural:
        return AdvancedPluralizationResult(plural, None, (), None, None, False)

    # right spelled plural
    if speller.spell(plural):
        return AdvancedPluralizationResult(plural, plural, (), None, None, True)

    # if no rightly spelled word can be found, use suggestions,
    # replacement of the endings and the Hunspell dictionary if
    # we can find something that is spelled correctly.
    suggestions = speller.suggest(plural)
    search_result:SearchResult = \
        search_by_suggestions(plural, suggestions) or \
        search_by_dictionary(speller, plural) or \
        search_by_dictionary_plus_s(speller, singular)
    
    if search_result:
        return AdvancedPluralizationResult(
            plural,
            search_result.plural,
            suggestions,
            search_result.switched_ending_from,
            search_result.switched_ending_to,
            True
        )

    return AdvancedPluralizationResult(plural, None, (), None, None, False)


def pluralize(singular: str, speller: Hunspell = None, ending_overrides: NounEndingMap = None) -> str:
    result = pluralize_advanced(singular, speller, ending_overrides)
    return result.plural



from typing import Dict

class NounEndingMap:

    def __init__(self, map: Dict[str, str] = None):
        self.__data = {}
        self.add_range(map)

    def add(self, singular_ending: str, plural_ending: str):
        self.__data[singular_ending] = plural_ending

    def add_range(self, map: Dict[str, str]):
        if map:
            for key in map.keys():
                self.__data[key] = map[key]

    def get_plural_map(self) -> Dict[str, str]:
        return self.__data

    def get_singular_map(self) -> Dict[str, str]:
        return {v: k for k, v in self.__data.items()}



# Yeah, sometimes is is too hard to do it algoritmically, or I'm
# just way too lazy, so here is a map for all those silly exceptions
# that we like to have in Dutch.
# Other words that end up here have plurals that are excepted by
# Hunspell, but not on VanDale.nl

from ..mapping import NounEndingMap

__hard_map = NounEndingMap({
    "gelid": "gelederen",
    "lid": "leden",
    "schip": "schepen",
    "stad": "steden",
    "infuus": "infusen",
    "lelie": "lelies",
    "oom": "ooms",
    "bruidegom": "bruidegoms",
    "serum": "serums",
    "make-up": "make-ups",
    "glas": "glazen",
    "graf": "graven",  # added case for singlar
    "bh": "bh's",
})

__only_for_singularize = NounEndingMap({
    "ine": "inen"
})


def pluralize_by_hard_map(singular: str, overrides: NounEndingMap) -> str:

    for nouns in [overrides, __hard_map]:
        if nouns:
            map = nouns.get_plural_map()
            for key in map:
                if singular.endswith(key):
                    return singular[0:0-len(key)] + map[key]

    return None


def singularize_by_hard_map(plural: str, overrides: NounEndingMap) -> str:

    for nouns in [overrides, __hard_map, __only_for_singularize]:
        if nouns:
            map = nouns.get_singular_map()
            for key in map:
                if plural.endswith(key):
                    return plural[0:0-len(key)] + map[key]

    return None



import re

from .helpers import VOWELS

__r_bastard_endings = re.compile("(é|eau|ey|ay|ieu|oe|ui)$")

def pluralize_bastard(singular: str) -> str:

    # dominee, abonnee
    if singular.endswith("nee") and not singular.endswith("snee"):
        return singular + "s"

    if singular.endswith("e") and singular[-2] not in VOWELS:
        return singular + "s"

    if __r_bastard_endings.search(singular):
        return singular + "s"

    return None



import re
from re import Pattern
from typing import AnyStr


def create_ends_with_regex(*args: str) -> Pattern:
    s = "|".join(map(re.escape, args))
    s = f"({s})$"
    return re.compile(s)


# aas, oos, ees, uus
__r_Xs = re.compile('(aa|oo|ee|uu)s$')
__r_XXs = re.compile('(oe|eu|ui|ij|ou|au)s$')

__s_replacement_exceptions = create_ends_with_regex(
    "eis", "mens", "kans", "prins", "wens", "dans", 
    "kers", "tendens", "kaars", "fiets")

__closed_to_open = create_ends_with_regex(
    "bad", "bevel", "dag", "drag", "dak", "gebed",
    "gat", "gebrek", "glaz", "god", "grav", "hertog",
    "oorlog", "schot", "vat", "slag", "weg")

VOWELS = ['a', 'e', 'i', 'o', 'u']


def change_last_character(singular: str) -> str:

    if not singular:
        return singular

    # exclude f-plural
    if singular.endswith("f") and \
            not singular.endswith("graaf") and \
            not singular.endswith("soof") and \
            not singular.endswith("scaaf") and \
            not singular in ['cenotaaf', 'paraaf']:
        singular = singular[0:-1] + "v"

    elif __r_Xs.search(singular):
        return singular[0:-2] + "z"

    elif __r_XXs.search(singular):
        return singular[0:-1] + "z"

    elif singular.endswith("s") and \
            singular[-2] not in VOWELS and \
            not __s_replacement_exceptions.search(singular):
        singular = singular[0:-1] + "z"

    if __closed_to_open.search(singular) or len(singular) < 3:
        return singular

    if singular[-1] not in VOWELS and \
            singular[-2] in VOWELS and \
            singular[-3] not in VOWELS:
        return singular + singular[-1]

    if singular[-1] not in VOWELS and \
            singular[-2] in VOWELS and \
            singular[-3] == singular[-2]:
        return singular[0:-2] + singular[-1]

    return singular



import inspect
import os
from pathlib import Path

from hunspell import Hunspell

h_nl = None
h_loaded = False


def __resolve_path(sub: str) -> str:
    base_path = Path(__resolve_path.__code__.co_filename)
    path = (base_path / sub).resolve()
    return str(path)


def __read_relative_path(sub: str)-> [str]:
    file = __resolve_path(sub)
    with open(file, encoding="utf-8-sig") as f:
        return f.read().splitlines()


def get_plural_nouns() -> [str]:
    return __read_relative_path("../dict/nouns-meervouden.txt")


def get_basic_words() -> [str]:
    return __read_relative_path("../dict/basiswoorden-gekeurd.txt")


def ensure_hunspell_nl() -> Hunspell:

    global h_nl
    global h_loaded

    if h_loaded: return h_nl
    
    h_nl = new_hunspell_nl()
    h_loaded = True
    return h_nl

def new_hunspell_nl() -> Hunspell:

    dictionary_path = __resolve_path("../dict/")
    hnspl = Hunspell("nl-nl", hunspell_data_dir=str(dictionary_path))

    # add words that are not present in current dictionary
    for list in [get_plural_nouns(), get_basic_words()]:
        for word in list:
            if not hnspl.spell(word):
                hnspl.add(word)

    return hnspl



import re

from .helpers import change_last_character

__long_vowel_endings_with_no_preceeding_vowel = re.compile("(?<![aeiou])(a|o|u|i|y)$")

def pluralize_long_vowel(singular: str) -> str:
    
    if __long_vowel_endings_with_no_preceeding_vowel.search(singular):
        return singular + "'s"

    return None



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="dutch-pluralizer",
    version="0.0.28",
    description="Generates Dutch plural and singular nouns in a very imperfect way using Hunspell dictionaries. Why imperfect? Because the Dutch language is full of exceptions.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/KeesCBakker/dutch-pluralizer-py",
    author="Kees C. Bakker / KeesTalksTech",
    author_email="info@keestalkstech.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["dutch_pluralizer"],
    include_package_data=True,
    install_requires=["cython", "cyhunspell"],
    entry_points={
        "console_scripts": [
            "dutch_pluralizer=dutch_pluralizer.__main__:main",
        ]
    },
)



from typing import Tuple, Union
from hunspell import Hunspell

__ending_pairs = [{
    "sen": "zen",
    "zen": "sen",
    "des": "den",
    "den": "des",
    "iën": "ieën",
    "ieën": "iën",
    "a": "ums",
    "i": "ussen"
}]


class SearchResult:
    def __init__(self, plural: str, switched_ending_from: str, switched_ending_to: str):
        self.plural = plural
        self.switched_ending_from = switched_ending_from
        self.switched_ending_to = switched_ending_to


def check_switch_ending(e1, e2, plural, suggestions):
    if plural.endswith(e1):
        plural = plural[0:0-len(e1)] + e2
        if plural in suggestions:
            return plural
    return None


def search_by_suggestions(plural:str, suggestions: Tuple[str]) -> Union[None, SearchResult]:
    for e in __ending_pairs:
        for key in e.keys():
            x = check_switch_ending(key, e[key], plural, suggestions)
            if x:
                return SearchResult(x, key, e[key])
                
    return None


def search_by_dictionary(speller:Hunspell, plural: str) -> Union[None, SearchResult]:
    for e in __ending_pairs:
        for key in e.keys():
            if plural.endswith(key):
                suggestion = plural[0:0-len(key)] + e[key]
                if speller.spell(suggestion):
                    return SearchResult(suggestion, key, e[key])

    return None


def search_by_dictionary_plus_s(speller: Hunspell, singular: str) -> Union[None, SearchResult]:

    plural = singular + 's'

    if speller.spell(plural):
        return SearchResult(plural, None,"s")

    return None
